/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This file is part of SableCC.                             *
 * See the file "LICENSE" for copyright information and the  *
 * terms and conditions for copying, distribution and        *
 * modification of SableCC.                                  *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

package org.sablecc.sablecc;

import java.util.List;
import java.util.Map;

import org.sablecc.sablecc.analysis.DepthFirstAdapter;
import org.sablecc.sablecc.node.*;

public class AltTransformAdapter extends DepthFirstAdapter
{
  ResolveAltIds altIds;
  ResolveTransformIds transformIds;
  String currentNewAltName;

  private Map<String, AElem> isElementIsAlist;

  private List<Node> listSimpleTermTransform;
  private Map<Node, String> simpleTermTransform;
  private Map<Node, String> simpleTermOrsimpleListTermTypes;

  AltTransformAdapter(Map<Node, String> simpleTermTransform,
                      List<Node> listSimpleTermTransform,
                      String currentNewAltName,
                      ResolveAltIds altIds,
                      ResolveTransformIds transformIds,
                      Map<String, AElem> isElementIsAlist,
                      Map<Node, String> simpleTermOrsimpleListTermTypes)
  {
    this.currentNewAltName = currentNewAltName;
    this.altIds = altIds;
    this.transformIds = transformIds;
    this.isElementIsAlist = isElementIsAlist;
    this.listSimpleTermTransform = listSimpleTermTransform;
    this.simpleTermTransform = simpleTermTransform;
    this.simpleTermOrsimpleListTermTypes = simpleTermOrsimpleListTermTypes;
  }

  @Override
  public void inASimpleTerm(ASimpleTerm node)
  {
    String name = node.getId().getText();

    if( !altIds.alts_elems.get(currentNewAltName).contains(name) &&
        !altIds.alts_elems.get(currentNewAltName).contains("$"+name) )
    {
      node.replaceBy( new ANullTerm() );
    }

    AElem elemNode = isElementIsAlist.get(currentNewAltName + name);
    if(elemNode != null)
    {
      TId simpleTermTail;
      if(node.getSimpleTermTail() != null)
      {
        simpleTermTail = node.getSimpleTermTail();
      }
      else
      {
        simpleTermTail = elemNode.getId().clone();
      }

      ASimpleTerm asimpleTerm = new ASimpleTerm( node.getSpecifier(), node.getId(), simpleTermTail);

      if(simpleTermOrsimpleListTermTypes.get(node) != null)
      {
        simpleTermOrsimpleListTermTypes.put(asimpleTerm, simpleTermOrsimpleListTermTypes.get(node));
      }

      node.replaceBy(asimpleTerm);
      String untransformedElementType = altIds.ids.elemTypes.get(elemNode);
      String key = untransformedElementType + "." + simpleTermTail.getText();
      String elementType = transformIds.getProdTransformIds().prodTransformElemTypesString.get(key);
      if(elementType == null)
      {
        // This should only happen for plain token types
        assert untransformedElementType.startsWith("T");
        elementType = untransformedElementType;
      }
      simpleTermTransform.put(asimpleTerm, "L" + elementType);

      //Terms are added here only if they were implicitely transformed
      listSimpleTermTransform.add( asimpleTerm );
    }
  }

  @Override
  public void inASimpleListTerm(ASimpleListTerm node)
  {
    String name = node.getId().getText();

    if( !altIds.alts_elems.get(currentNewAltName).contains(name) &&
        !altIds.alts_elems.get(currentNewAltName).contains("$"+name) )
    {
      node.replaceBy( null );
    }

    AElem elemNode = isElementIsAlist.get(currentNewAltName + name);
    if(elemNode != null)
    {
      TId simpleTermTail;
      if(node.getSimpleTermTail() != null)
      {
        simpleTermTail = node.getSimpleTermTail();
      }
      else
      {
        simpleTermTail = elemNode.getId().clone();
      }

      TId tid;
      tid = ( altIds.alts_elems_list_elemName.get(currentNewAltName).contains(name) ?
              node.getId() : new TId( "$" + node.getId().getText() ) );

      ASimpleListTerm asimpleListTerm = new ASimpleListTerm( node.getSpecifier(), tid, simpleTermTail);

      if(simpleTermOrsimpleListTermTypes.get(node) != null)
      {
        simpleTermOrsimpleListTermTypes.put(asimpleListTerm, simpleTermOrsimpleListTermTypes.get(node));
      }

      node.replaceBy(asimpleListTerm);
      String untransformedElementType = altIds.ids.elemTypes.get(elemNode);
      String key = untransformedElementType + "." + simpleTermTail.getText();
      String elementType = transformIds.getProdTransformIds().prodTransformElemTypesString.get(key);
      if(elementType == null)
      {
        // This should only happen for plain token types
        assert untransformedElementType.startsWith("T");
        elementType = untransformedElementType;
      }
      simpleTermTransform.put(asimpleListTerm, "L" + elementType);

      //Terms are added here only if they were implicitely transformed
      listSimpleTermTransform.add( asimpleListTerm );
    }
  }

  @Override
  public void outAListTerm(AListTerm node)
  {
    if( (node.getListTerms() != null) && (node.getListTerms().size() > 0) )
    {
      Object[] temp = node.getListTerms().toArray();

      if(simpleTermTransform.get(temp[0]) != null)
      {
        String firstTermType = simpleTermTransform.get(temp[0]);

        if(firstTermType != null)
        {
          if(!firstTermType.startsWith("L"))
          {
            simpleTermTransform.put(node, "L" + firstTermType);
          }
          else
          {
            simpleTermTransform.put(node, firstTermType);
          }
        }
      }
    }
  }

}
